@@ -232,7 +232,8 @@ fetch_ibus_engines_result (GObject            *object,
                                                    result,
                                                    &error);
         if (!list && error) {
-                g_warning ("Couldn't finish IBus request: %s", error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Couldn't finish IBus request: %s", error->message);
                 g_error_free (error);
 
                 clear_ibus (manager);
@@ -413,7 +414,8 @@ set_ibus_engine_finish (GObject            *object,
 
         result = ibus_bus_set_global_engine_async_finish (ibus, res, &error);
         if (!result) {
-                g_warning ("Couldn't set IBus engine: %s", error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Couldn't set IBus engine: %s", error->message);
                 g_error_free (error);
         }
 }
@@ -549,8 +551,21 @@ engine_from_locale (void)
 {
         const gchar *locale;
         const gchar *locale_engine[][2] = {
+                { "as_IN", "m17n:as:phonetic" },
+                { "bn_IN", "m17n:bn:inscript" },
+                { "gu_IN", "m17n:gu:inscript" },
+                { "hi_IN", "m17n:hi:inscript" },
                 { "ja_JP", "anthy" },
+                { "kn_IN", "m17n:kn:kgp" },
                 { "ko_KR", "hangul" },
+                { "mai_IN", "m17n:mai:inscript" },
+                { "ml_IN", "m17n:ml:inscript" },
+                { "mr_IN", "m17n:mr:inscript" },
+                { "or_IN", "m17n:or:inscript" },
+                { "pa_IN", "m17n:pa:inscript" },
+                { "sd_IN", "m17n:sd:inscript" },
+                { "ta_IN", "m17n:ta:tamil99" },
+                { "te_IN", "m17n:te:inscript" },
                 { "zh_CN", "pinyin" },
                 { "zh_HK", "cangjie3" },
                 { "zh_TW", "chewing" },
@@ -1296,10 +1311,11 @@ settings_changed (GSettings          *settings,
 	    g_strcmp0 (key, KEY_BELL_MODE) == 0) {
 		g_debug ("Bell setting '%s' changed, applying bell settings", key);
 		apply_bell (manager);
-	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0 ||
-		 g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
-		g_debug ("Num-Lock state '%s' changed, applying num-lock state settings", key);
+	} else if (g_strcmp0 (key, KEY_REMEMBER_NUMLOCK_STATE) == 0) {
+		g_debug ("Remember Num-Lock state '%s' changed, applying num-lock settings", key);
 		apply_numlock (manager);
+	} else if (g_strcmp0 (key, KEY_NUMLOCK_STATE) == 0) {
+		g_debug ("Num-Lock state '%s' changed, will apply at next startup", key);
 	} else if (g_strcmp0 (key, KEY_REPEAT) == 0 ||
 		 g_strcmp0 (key, KEY_INTERVAL) == 0 ||
 		 g_strcmp0 (key, KEY_DELAY) == 0) {
