@@ -392,7 +392,7 @@ grab_media_key (MediaKey            *key,
 
 	if (key->key != NULL) {
 		need_flush = TRUE;
-		grab_key_unsafe (key->key, FALSE, manager->priv->screens);
+		ungrab_key_unsafe (key->key, manager->priv->screens);
 	}
 
 	free_key (key->key);
@@ -407,7 +407,7 @@ grab_media_key (MediaKey            *key,
 		return need_flush;
 	}
 
-	grab_key_unsafe (key->key, TRUE, manager->priv->screens);
+	grab_key_unsafe (key->key, GSD_KEYGRAB_NORMAL, manager->priv->screens);
 
 	g_free (tmp);
 
@@ -507,9 +507,8 @@ update_custom_binding (GsdMediaKeysManager *manager,
                         if (key->key) {
                                 gdk_error_trap_push ();
 
-                                grab_key_unsafe (key->key,
-                                                 FALSE,
-                                                 manager->priv->screens);
+                                ungrab_key_unsafe (key->key,
+                                                   manager->priv->screens);
 
                                 gdk_flush ();
                                 if (gdk_error_trap_pop ())
@@ -587,9 +586,8 @@ gsettings_custom_changed_cb (GSettings           *settings,
                 if (key->key) {
                         gdk_error_trap_push ();
 
-                        grab_key_unsafe (key->key,
-                                         FALSE,
-                                         manager->priv->screens);
+                        ungrab_key_unsafe (key->key,
+                                           manager->priv->screens);
 
                         gdk_flush ();
                         if (gdk_error_trap_pop ())
@@ -2299,7 +2297,7 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
                         key = g_ptr_array_index (manager->priv->keys, i);
 
                         if (key->key)
-                                grab_key_unsafe (key->key, FALSE, priv->screens);
+                                ungrab_key_unsafe (key->key, priv->screens);
                 }
                 g_ptr_array_free (priv->keys, TRUE);
                 priv->keys = NULL;
